package com.greenorange.promotion.model.dto.project;

import com.greenorange.promotion.annotation.ProjectStatusEnumValue;
import com.greenorange.promotion.model.enums.ProjectStatusEnum;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.Min;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;

/**
 * 项目状态更新请求体
 */
@Data
@Schema(description = "项目状态更新请求体", requiredProperties = {"id", "projectStatus"})
public class ProjectStatusUpdateRequest implements Serializable {

    /**
     * 项目 ID
     */
    @Min(value = 1L, message = "项目 ID不能小于1")
    @Schema(description = "项目 ID", example = "1")
    private Long id;


    /**
     * 项目状态[项目运行(running)|人数已满(full)|项目暂停(paused)]
     */
    @ProjectStatusEnumValue(enumClass = ProjectStatusEnum.class)
    @Schema(description = "项目状态[项目运行(running)|人数已满(full)|项目暂停(paused)]", example = "running")
    private String projectStatus;


    @Serial
    private static final long serialVersionUID = 1L;
}
