package com.greenorange.promotion.model.dto.projectNotification;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import com.greenorange.promotion.common.PageRequest;

/**
 * 项目通知查询请求体，继承自分页请求 PageRequest
 */
@Data
@Schema(description = "项目通知查询请求体", requiredProperties = {"current", "pageSize"})
public class ProjectNotificationQueryRequest extends PageRequest implements Serializable {

    /**
     * 通知ID
     */
    @Min(value = 1L, message = "通知ID ID不能小于1")
    @Schema(description = "通知ID", example = "1")
    private Long id;

    /**
     * 通知标题
     */
    @Schema(description = "通知标题", example = "美团试吃官-涨价通知")
    private String notificationTitle;

    /**
     * 通知内容
     */
    @Schema(description = "通知内容", example = "富文本")
    private String notificationContent;

    /**
     * 项目ID
     */
    @Min(value = 1L, message = "项目ID ID不能小于1")
    @Schema(description = "项目ID", example = "1")
    private Long projectId;


    @Serial
    private static final long serialVersionUID = 1L;
}

