package com.greenorange.promotion.model.dto.userProject;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import com.greenorange.promotion.common.PageRequest;

/**
 * 用户项目查询请求体，继承自分页请求 PageRequest
 */
@Data
@Schema(description = "用户项目查询请求体", requiredProperties = {"current", "pageSize"})
public class UserProjectQueryRequest extends PageRequest implements Serializable {

    /**
     * 用户项目ID
     */
    @Min(value = 1L, message = "用户项目ID ID不能小于1")
    @Schema(description = "用户项目ID", example = "1")
    private Long id;

    /**
     * 项目ID
     */
    @Min(value = 1L, message = "项目ID ID不能小于1")
    @Schema(description = "项目ID", example = "1")
    private Long projectId;

    /**
     * 项目名称
     */
    @Schema(description = "项目名称", example = "美团省钱包")
    private String projectName;

    /**
     * 项目图片URL
     */
    @Schema(description = "项目图片URL", example = "3E8U2AM8")
    private String projectImage;

    /**
     * 项目结算周期
     */
    @Schema(description = "项目结算周期", example = "2")
    private Integer projectSettlementCycle;

    /**
     * 用户ID
     */
    @Min(value = 1L, message = "用户ID ID不能小于1")
    @Schema(description = "用户ID", example = "1")
    private Long userId;


    @Serial
    private static final long serialVersionUID = 1L;
}

